@@ -16,8 +16,8 @@ use crate::lib::std::ops::{Range, RangeFrom, RangeTo};
 use crate::traits::{AsChar, InputIter, InputLength, InputTakeAtPosition, ParseTo};
 use crate::traits::{Compare, CompareResult, Offset, Slice};
 
-#[macro_use]
-mod macros;
+#[cfg(test)]
+mod tests;
 
 /// Return the remaining input.
 ///
@@ -71,26 +71,17 @@ where
 /// assert_eq!(parser.parse("abc"), Err(Err::Error(("abc", ErrorKind::Digit))));
 /// # }
 /// ```
-pub fn map<I, O1, O2, E, F, G>(mut first: F, mut second: G) -> impl FnMut(I) -> IResult<I, O2, E>
+pub fn map<I, O1, O2, E, F, G>(mut parser: F, mut f: G) -> impl FnMut(I) -> IResult<I, O2, E>
 where
   F: Parser<I, O1, E>,
   G: FnMut(O1) -> O2,
 {
   move |input: I| {
-    let (input, o1) = first.parse(input)?;
-    Ok((input, second(o1)))
+    let (input, o1) = parser.parse(input)?;
+    Ok((input, f(o1)))
   }
 }
 
-#[doc(hidden)]
-pub fn mapc<I, O1, O2, E, F, G>(input: I, first: F, second: G) -> IResult<I, O2, E>
-where
-  F: Fn(I) -> IResult<I, O1, E>,
-  G: Fn(O1) -> O2,
-{
-  map(first, second).parse(input)
-}
-
 /// Applies a function returning a `Result` over the result of a parser.
 ///
 /// ```rust
@@ -113,8 +104,8 @@ where
 /// # }
 /// ```
 pub fn map_res<I: Clone, O1, O2, E: FromExternalError<I, E2>, E2, F, G>(
-  mut first: F,
-  mut second: G,
+  mut parser: F,
+  mut f: G,
 ) -> impl FnMut(I) -> IResult<I, O2, E>
 where
   F: Parser<I, O1, E>,
@@ -122,27 +113,14 @@ where
 {
   move |input: I| {
     let i = input.clone();
-    let (input, o1) = first.parse(input)?;
-    match second(o1) {
+    let (input, o1) = parser.parse(input)?;
+    match f(o1) {
       Ok(o2) => Ok((input, o2)),
       Err(e) => Err(Err::Error(E::from_external_error(i, ErrorKind::MapRes, e))),
     }
   }
 }
 
-#[doc(hidden)]
-pub fn map_resc<I: Clone, O1, O2, E: FromExternalError<I, E2>, E2, F, G>(
-  input: I,
-  first: F,
-  second: G,
-) -> IResult<I, O2, E>
-where
-  F: Fn(I) -> IResult<I, O1, E>,
-  G: Fn(O1) -> Result<O2, E2>,
-{
-  map_res(first, second)(input)
-}
-
 /// Applies a function returning an `Option` over the result of a parser.
 ///
 /// ```rust
@@ -165,8 +143,8 @@ where
 /// # }
 /// ```
 pub fn map_opt<I: Clone, O1, O2, E: ParseError<I>, F, G>(
-  mut first: F,
-  mut second: G,
+  mut parser: F,
+  mut f: G,
 ) -> impl FnMut(I) -> IResult<I, O2, E>
 where
   F: Parser<I, O1, E>,
@@ -174,27 +152,14 @@ where
 {
   move |input: I| {
     let i = input.clone();
-    let (input, o1) = first.parse(input)?;
-    match second(o1) {
+    let (input, o1) = parser.parse(input)?;
+    match f(o1) {
       Some(o2) => Ok((input, o2)),
       None => Err(Err::Error(E::from_error_kind(i, ErrorKind::MapOpt))),
     }
   }
 }
 
-#[doc(hidden)]
-pub fn map_optc<I: Clone, O1, O2, E: ParseError<I>, F, G>(
-  input: I,
-  first: F,
-  second: G,
-) -> IResult<I, O2, E>
-where
-  F: Fn(I) -> IResult<I, O1, E>,
-  G: Fn(O1) -> Option<O2>,
-{
-  map_opt(first, second)(input)
-}
-
 /// Applies a parser over the result of another one.
 ///
 /// ```rust
@@ -213,33 +178,20 @@ where
 /// # }
 /// ```
 pub fn map_parser<I, O1, O2, E: ParseError<I>, F, G>(
-  mut first: F,
-  mut second: G,
+  mut parser: F,
+  mut applied_parser: G,
 ) -> impl FnMut(I) -> IResult<I, O2, E>
 where
   F: Parser<I, O1, E>,
   G: Parser<O1, O2, E>,
 {
   move |input: I| {
-    let (input, o1) = first.parse(input)?;
-    let (_, o2) = second.parse(o1)?;
+    let (input, o1) = parser.parse(input)?;
+    let (_, o2) = applied_parser.parse(o1)?;
     Ok((input, o2))
   }
 }
 
-#[doc(hidden)]
-pub fn map_parserc<I, O1, O2, E: ParseError<I>, F, G>(
-  input: I,
-  first: F,
-  second: G,
-) -> IResult<I, O2, E>
-where
-  F: Fn(I) -> IResult<I, O1, E>,
-  G: Fn(O1) -> IResult<O1, O2, E>,
-{
-  map_parser(first, second)(input)
-}
-
 /// Creates a new parser from the output of the first parser, then apply that parser over the rest of the input.
 ///
 /// ```rust
@@ -257,8 +209,8 @@ where
 /// # }
 /// ```
 pub fn flat_map<I, O1, O2, E: ParseError<I>, F, G, H>(
-  mut first: F,
-  second: G,
+  mut parser: F,
+  applied_parser: G,
 ) -> impl FnMut(I) -> IResult<I, O2, E>
 where
   F: Parser<I, O1, E>,
@@ -266,8 +218,8 @@ where
   H: Parser<I, O2, E>,
 {
   move |input: I| {
-    let (input, o1) = first.parse(input)?;
-    second(o1).parse(input)
+    let (input, o1) = parser.parse(input)?;
+    applied_parser(o1).parse(input)
   }
 }
 
@@ -302,14 +254,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn optc<I: Clone, O, E: ParseError<I>, F>(input: I, f: F) -> IResult<I, Option<O>, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  opt(f)(input)
-}
-
 /// Calls the parser if the condition is met.
 ///
 /// ```rust
@@ -348,14 +292,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn condc<I, O, E: ParseError<I>, F>(input: I, b: bool, f: F) -> IResult<I, Option<O>, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  cond(b, f)(input)
-}
-
 /// Tries to apply its parser without consuming the input.
 ///
 /// ```rust
@@ -384,14 +320,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn peekc<I: Clone, O, E: ParseError<I>, F>(input: I, f: F) -> IResult<I, O, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  peek(f)(input)
-}
-
 /// returns its input if it is at the end of input data
 ///
 /// When we're at the end of the data, this combinator
@@ -446,14 +374,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn completec<I: Clone, O, E: ParseError<I>, F>(input: I, f: F) -> IResult<I, O, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  complete(f)(input)
-}
-
 /// Succeeds if all the input has been consumed by its child parser.
 ///
 /// ```rust
@@ -526,21 +446,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn verifyc<I: Clone, O1, O2, E: ParseError<I>, F, G>(
-  input: I,
-  first: F,
-  second: G,
-) -> IResult<I, O1, E>
-where
-  F: Fn(I) -> IResult<I, O1, E>,
-  G: Fn(&O2) -> bool,
-  O1: Borrow<O2>,
-  O2: ?Sized,
-{
-  verify(first, second)(input)
-}
-
 /// Returns the provided value if the child parser succeeds.
 ///
 /// ```rust
@@ -566,18 +471,6 @@ where
   move |input: I| parser.parse(input).map(|(i, _)| (i, val.clone()))
 }
 
-#[doc(hidden)]
-pub fn valuec<I, O1: Clone, O2, E: ParseError<I>, F>(
-  input: I,
-  val: O1,
-  parser: F,
-) -> IResult<I, O1, E>
-where
-  F: Fn(I) -> IResult<I, O2, E>,
-{
-  value(val, parser)(input)
-}
-
 /// Succeeds if the child parser returns an error.
 ///
 /// ```rust
@@ -607,14 +500,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn notc<I: Clone, O, E: ParseError<I>, F>(input: I, parser: F) -> IResult<I, (), E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  not(parser)(input)
-}
-
 /// If the child parser was successful, return the consumed input as produced value.
 ///
 /// ```rust
@@ -649,17 +534,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn recognizec<I: Clone + Offset + Slice<RangeTo<usize>>, O, E: ParseError<I>, F>(
-  input: I,
-  parser: F,
-) -> IResult<I, I, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  recognize(parser)(input)
-}
-
 /// if the child parser was successful, return the consumed input with the output
 /// as a tuple. Functions similarly to [recognize](fn.recognize.html) except it
 /// returns the parser output as well.
@@ -717,16 +591,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn consumedc<I, O, E: ParseError<I>, F>(input: I, parser: F) -> IResult<I, (I, O), E>
-where
-  I: Clone + Offset + Slice<RangeTo<usize>>,
-  E: ParseError<E>,
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  consumed(parser)(input)
-}
-
 /// transforms an error to failure
 ///
 /// ```rust
@@ -752,14 +616,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn cutc<I, O, E: ParseError<I>, F>(input: I, parser: F) -> IResult<I, O, E>
-where
-  F: Fn(I) -> IResult<I, O, E>,
-{
-  cut(parser)(input)
-}
-
 /// automatically converts the child parser's result to another type
 ///
 /// it will be able to convert the output value and the error value
@@ -800,18 +656,6 @@ where
   }
 }
 
-#[doc(hidden)]
-pub fn intoc<I, O1, O2, E1, E2, F>(input: I, parser: F) -> IResult<I, O2, E2>
-where
-  O1: Into<O2>,
-  E1: Into<E2>,
-  E1: ParseError<I>,
-  E2: ParseError<I>,
-  F: Parser<I, O1, E1>,
-{
-  into(parser)(input)
-}
-
 /// Creates an iterator from input data and a parser.
 ///
 /// Call the iterator's [ParserIterator::finish] method to get the remaining input if successful,
@@ -929,208 +773,15 @@ pub fn success<I, O: Clone, E: ParseError<I>>(val: O) -> impl Fn(I) -> IResult<I
   move |input: I| Ok((input, val.clone()))
 }
 
-#[cfg(test)]
-mod tests {
-  use super::*;
-  use crate::bytes::complete::take;
-  use crate::error::ParseError;
-  use crate::internal::{Err, IResult, Needed};
-  use crate::number::complete::u8;
-
-  macro_rules! assert_parse(
-    ($left: expr, $right: expr) => {
-      let res: $crate::IResult<_, _, (_, ErrorKind)> = $left;
-      assert_eq!(res, $right);
-    };
-  );
-
-  /*#[test]
-  fn t1() {
-    let v1:Vec<u8> = vec![1,2,3];
-    let v2:Vec<u8> = vec![4,5,6];
-    let d = Ok((&v1[..], &v2[..]));
-    let res = d.flat_map(print);
-    assert_eq!(res, Ok((&v2[..], ())));
-  }*/
-
-  #[test]
-  fn eof_on_slices() {
-    let not_over: &[u8] = &b"Hello, world!"[..];
-    let is_over: &[u8] = &b""[..];
-
-    let res_not_over = eof(not_over);
-    assert_parse!(
-      res_not_over,
-      Err(Err::Error(error_position!(not_over, ErrorKind::Eof)))
-    );
-
-    let res_over = eof(is_over);
-    assert_parse!(res_over, Ok((is_over, is_over)));
-  }
-
-  #[test]
-  fn eof_on_strs() {
-    let not_over: &str = "Hello, world!";
-    let is_over: &str = "";
-
-    let res_not_over = eof(not_over);
-    assert_parse!(
-      res_not_over,
-      Err(Err::Error(error_position!(not_over, ErrorKind::Eof)))
-    );
-
-    let res_over = eof(is_over);
-    assert_parse!(res_over, Ok((is_over, is_over)));
-  }
-
-  /*
-  #[test]
-  fn end_of_input() {
-      let not_over = &b"Hello, world!"[..];
-      let is_over = &b""[..];
-      named!(eof_test, eof!());
-
-      let res_not_over = eof_test(not_over);
-      assert_eq!(res_not_over, Err(Err::Error(error_position!(not_over, ErrorKind::Eof))));
-
-      let res_over = eof_test(is_over);
-      assert_eq!(res_over, Ok((is_over, is_over)));
-  }
-  */
-
-  #[test]
-  fn rest_on_slices() {
-    let input: &[u8] = &b"Hello, world!"[..];
-    let empty: &[u8] = &b""[..];
-    assert_parse!(rest(input), Ok((empty, input)));
-  }
-
-  #[test]
-  fn rest_on_strs() {
-    let input: &str = "Hello, world!";
-    let empty: &str = "";
-    assert_parse!(rest(input), Ok((empty, input)));
-  }
-
-  #[test]
-  fn rest_len_on_slices() {
-    let input: &[u8] = &b"Hello, world!"[..];
-    assert_parse!(rest_len(input), Ok((input, input.len())));
-  }
-
-  use crate::lib::std::convert::From;
-  impl From<u32> for CustomError {
-    fn from(_: u32) -> Self {
-      CustomError
-    }
-  }
-
-  impl<I> ParseError<I> for CustomError {
-    fn from_error_kind(_: I, _: ErrorKind) -> Self {
-      CustomError
-    }
-
-    fn append(_: I, _: ErrorKind, _: CustomError) -> Self {
-      CustomError
-    }
-  }
-
-  struct CustomError;
-  #[allow(dead_code)]
-  fn custom_error(input: &[u8]) -> IResult<&[u8], &[u8], CustomError> {
-    //fix_error!(input, CustomError, alphanumeric)
-    crate::character::streaming::alphanumeric1(input)
-  }
-
-  #[test]
-  fn test_flat_map() {
-    let input: &[u8] = &[3, 100, 101, 102, 103, 104][..];
-    assert_parse!(
-      flat_map(u8, take)(input),
-      Ok((&[103, 104][..], &[100, 101, 102][..]))
-    );
-  }
-
-  #[test]
-  fn test_map_opt() {
-    let input: &[u8] = &[50][..];
-    assert_parse!(
-      map_opt(u8, |u| if u < 20 { Some(u) } else { None })(input),
-      Err(Err::Error((&[50][..], ErrorKind::MapOpt)))
-    );
-    assert_parse!(
-      map_opt(u8, |u| if u > 20 { Some(u) } else { None })(input),
-      Ok((&[][..], 50))
-    );
-  }
-
-  #[test]
-  fn test_map_parser() {
-    let input: &[u8] = &[100, 101, 102, 103, 104][..];
-    assert_parse!(
-      map_parser(take(4usize), take(2usize))(input),
-      Ok((&[104][..], &[100, 101][..]))
-    );
-  }
-
-  #[test]
-  fn test_all_consuming() {
-    let input: &[u8] = &[100, 101, 102][..];
-    assert_parse!(
-      all_consuming(take(2usize))(input),
-      Err(Err::Error((&[102][..], ErrorKind::Eof)))
-    );
-    assert_parse!(
-      all_consuming(take(3usize))(input),
-      Ok((&[][..], &[100, 101, 102][..]))
-    );
-  }
-
-  #[test]
-  #[allow(unused)]
-  fn test_verify_ref() {
-    use crate::bytes::complete::take;
-
-    let mut parser1 = verify(take(3u8), |s: &[u8]| s == &b"abc"[..]);
-
-    assert_eq!(parser1(&b"abcd"[..]), Ok((&b"d"[..], &b"abc"[..])));
-    assert_eq!(
-      parser1(&b"defg"[..]),
-      Err(Err::Error((&b"defg"[..], ErrorKind::Verify)))
-    );
-
-    fn parser2(i: &[u8]) -> IResult<&[u8], u32> {
-      verify(crate::number::streaming::be_u32, |val: &u32| *val < 3)(i)
-    }
-  }
-
-  #[test]
-  #[cfg(feature = "alloc")]
-  fn test_verify_alloc() {
-    use crate::bytes::complete::take;
-    let mut parser1 = verify(map(take(3u8), |s: &[u8]| s.to_vec()), |s: &[u8]| {
-      s == &b"abc"[..]
-    });
-
-    assert_eq!(parser1(&b"abcd"[..]), Ok((&b"d"[..], (&b"abc").to_vec())));
-    assert_eq!(
-      parser1(&b"defg"[..]),
-      Err(Err::Error((&b"defg"[..], ErrorKind::Verify)))
-    );
-  }
-
-  #[test]
-  #[cfg(feature = "std")]
-  fn test_into() {
-    use crate::bytes::complete::take;
-    use crate::{
-      error::{Error, ParseError},
-      Err,
-    };
-
-    let mut parser = into(take::<_, _, Error<_>>(3u8));
-    let result: IResult<&[u8], Vec<u8>> = parser(&b"abcdefg"[..]);
-
-    assert_eq!(result, Ok((&b"defg"[..], vec![97, 98, 99])));
-  }
+/// A parser which always fails.
+///
+/// ```rust
+/// # use nom::{Err, error::ErrorKind, IResult};
+/// use nom::combinator::fail;
+///
+/// let s = "string";
+/// assert_eq!(fail::<_, &str, _>(s), Err(Err::Error((s, ErrorKind::Fail))));
+/// ```
+pub fn fail<I, O, E: ParseError<I>>(i: I) -> IResult<I, O, E> {
+  Err(Err::Error(E::from_error_kind(i, ErrorKind::Fail)))
 }
